from flask import render_template, redirect, url_for, flash, Blueprint
from sqlalchemy.exc import OperationalError
from celery import states
from celery.utils.log import get_task_logger
from flask_admin.contrib.sqla import ModelView
from flask_login import login_required

from database import app, admin, db
from covid19.blueprints.application.application_services import owid_service
from covid19.blueprints.application.application_workers import celery
from covid19.blueprints.owid.owid_model import OwidDateReported, OwidData, OwidContinent, OwidCountry
from covid19.blueprints.owid.owid_model_import import OwidImport
from covid19.blueprints.application.application_model_transient import ApplicationPage


app_owid_test = Blueprint('owid_test', __name__, template_folder='templates', url_prefix='/owid/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_owid_test.route('/tests')
@login_required
def url_owid_test_tests():
    page_info = ApplicationPage('OWID', "Tests")
    return render_template(
        'owid_test/owid_tests.html',
        page_info=page_info)

@app_owid_test.route('/update_dimension_tables_only')
@login_required
def url_owid_test_update_dimension_tables_only():
    app.logger.info("test_update_dimension_tables_only - START")
    flash("test_update_dimension_tables_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))

@app_owid_test.route('/update_fact_table_incremental_only')
@login_required
def url_owid_test_update_fact_table_incremental_only():
    app.logger.info("update_fact_table_incremental_only - START")
    flash("update_fact_table_incremental_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))

@app_owid_test.route('/update_fact_table_initial_only')
@login_required
def url_owid_test_update_fact_table_initial_only():
    app.logger.info("update_fact_table_initial_only - START")
    flash("update_fact_table_initial_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))

@app_owid_test.route('/update_star_schema_incremental')
@login_required
def url_owid_test_update_star_schema_incremental():
    app.logger.info("update_star_schema_incremental - START")
    flash("update_star_schema_incremental - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))

@app_owid_test.route('/update_star_schema_initial')
@login_required
def url_owid_test_update_star_schema_initial():
    app.logger.info("update_star_schema_initial - START")
    flash("update_star_schema_initial - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))




