from flask import render_template, redirect, url_for, flash, Blueprint
from sqlalchemy.exc import OperationalError
from celery import states
from celery.utils.log import get_task_logger
from flask_admin.contrib.sqla import ModelView
from flask_login import login_required

from database import app, admin, db
from covid19.blueprints.application.application_services import who_service
from covid19.blueprints.application.application_workers import celery

from covid19.blueprints.who.who_model_import import WhoImport
from covid19.blueprints.who.who_model import WhoRegion, WhoCountry, WhoDateReported, WhoData
from covid19.blueprints.application.application_model_transient import ApplicationPage

from covid19.blueprints.who.who_views import app_who

from covid19.blueprints.who_test.who_test_service import WhoTestService

who_test_service = WhoTestService(db)

app_who_test = Blueprint('who_test', __name__, template_folder='templates', url_prefix='/who/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------


@app_who_test.route('/tests')
@login_required
def url_who_test_tests():
    page_info = ApplicationPage('WHO', "Tests")
    return render_template(
        'who_test/who_tests.html',
        page_info=page_info)

@app_who_test.route('/who_import/countries')
@login_required
def url_who_test_who_import_countries():
    flash("url_who_mytest - START: WhoImport.countries()")
    app.logger.info("url_who_mytest - START: WhoImport.countries()")
    i = 0
    for c in WhoImport.countries():
        i += 1
        line = " | " + str(i) + " | " + c.countries.country_code + " | " + c.countries.country + " | " + c.countries.who_region + " | "
        app.logger.info(line)
    flash("url_who_mytest - DONE: WhoImport.countries()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_import/get_new_dates_as_array')
@login_required
def url_who_test_who_import_get_new_dates_as_array():
    app.logger.info("url_who_mytest - DONE: WhoImport.countries()")
    flash("url_who_mytest - START: WhoImport.get_new_dates_as_array()")
    app.logger.info("url_who_mytest - START: WhoImport.get_new_dates_as_array()")
    app.logger.info("WhoImport.get_new_dates_as_array():")
    i = 0
    for date_reported in WhoImport.get_new_dates_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    joungest_datum = WhoDateReported.get_joungest_datum()
    app.logger.info("joungest_datum:")
    app.logger.info(joungest_datum)
    app.logger.info("WhoData.get_data_for_one_day(joungest_datum):")
    i = 0
    for who_data in WhoData.get_data_for_one_day(joungest_datum):
        i += 1
        line = " | " + str(i) + " | " + str(who_data.date_reported) + " | " + who_data.country.country + " | "
        app.logger.info(line)
    app.logger.info("WhoData.delete_data_for_one_day(joungest_datum)")
    WhoData.delete_data_for_one_day(joungest_datum)
    app.logger.info("WhoImport.get_new_dates_as_array():")
    i = 0
    for date_reported in WhoImport.get_new_dates_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    flash("url_who_mytest - DONE: WhoImport.get_new_dates_as_array()")
    app.logger.info("url_who_mytest - DONE: WhoImport.get_new_dates_as_array()")
    return redirect(url_for('who_test.url_who_test_tests'))

@app_who_test.route('/who_data/get_datum_of_all_who_data')
@login_required
def url_who_test_who_data_get_datum_of_all_who_data():
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    flash("url_who_test_who_data_get_datum_of_all_who_data - START: WhoData.get_datum_of_all_who_data()")
    for datum in WhoData.get_datum_of_all_who_data():
        app.logger.info(str(datum.date_reported))
    flash("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    return redirect(url_for('who_test.url_who_test_tests'))


